# SPDX-FileCopyrightText: 2024-present Alex Rudy <github@alexrudy.net>
#
# SPDX-License-Identifier: MIT
from ._version import __version__ as version

author = "Alex Rudy"
email = "opensource@alexrudy.net"

__all__ = ["author", "email", "version"]
